import { HttpsProxySocket } from './HttpsProxySocket';
import * as agentBase from 'agent-base';
import * as tls from 'tls';

/**
 * Construct an agent for http(s) requests. Mostly for testing purposes.
 *
 * @param proxy
 */
export function agent(proxy: HttpsProxySocket, options?: tls.ConnectionOptions) {
  return agentBase(async (req, opts: any) => {
    const socket = await proxy.connect(opts);

    if (opts.secureEndpoint) {
      // Upgrade to TLS
      let tlsOptions: tls.ConnectionOptions = {
        socket: socket,
        servername: opts.servername || opts.host
      };
      if (typeof opts.rejectUnauthorized != 'undefined') {
        // There's a difference between 'undefined' (equivalent of false) and "not set" (equivalent of true)
        tlsOptions.rejectUnauthorized = opts.rejectUnauthorized;
      }
      Object.assign(tlsOptions, options);
      const tlsSocket = tls.connect(tlsOptions);
      return tlsSocket;
    } else {
      return socket;
    }
  });
}
